#*****************************************************************************
#           PytronixGuiWin.py Python script for DOCSIS ATP Test Application
#
#                   Copyright 2007, Cisco Systems, Inc.
#
#                      Advanced Subscriber Networks
#                         5030 Sugarloaf Parkway
#                            P.O.Box 465447
#                       Lawrenceville, GA 30042
#
#                     Proprietary and Confidential
#         Unauthorized distribution or copying is prohibited
#                          All rights reserved
#
# No part of self computer software may be reprinted, reproduced or utilized
# in any form or by any electronic, mechanical, or other means, now known or
# hereafter invented, including photocopying and recording, or using any
# information storage and retrieval system, without permission in writing
# from Cisco Systems, Inc.
#
#*****************************************************************************

import os
import wx
import time
import logging
from pytronix.lib.general import NullLogHandler

#import PytronixLogging as logging
#import PytronixStyle

#################################################################################################
# Server Classes
# Taken fromm http://wiki.wxpython.org/AsynchronousSockets
import socket
import asyncore

global_gui = None

class DispatcherConnection(asyncore.dispatcher_with_send):
    inc_buffer = ''
    def __init__(self, connection, mainwindow, callback):
        self.mainwindow = mainwindow
        self.callback   = callback
        asyncore.dispatcher_with_send.__init__(self, connection)


    def writable(self):
        return bool(self.out_buffer)


    def handle_write(self):
        self.initiate_send()
    

    def handle_read(self):
        self.inc_buffer += self.recv(512)
        while '\n' in self.inc_buffer:
            snd, self.inc_buffer = self.inc_buffer.split('\n', 1)
            snd += '\n'
            exit,reply = self.callback( str(self.addr), 'tRead', snd )
            #self.log("Line from %s: %r"%(self.addr, snd))
            if reply:
                self.send(reply)
            if exit:
                self.close()
    

    def handle_close(self):
        self.callback( str(self.addr), 'tClose', '' )
        self.close()


class DispatcherServer(asyncore.dispatcher):
    def __init__(self, host, port, mainwindow, callback):
        self.mainwindow = mainwindow
        self.callback = callback
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.is_socke_open = False
        try:
            self.bind((host, port))
            self.is_socke_open = True
            self.listen(5)
        except:
            pass


    def handle_accept(self):
        connection, info = self.accept()
        self.callback( str(info), 'tOpen', '' )
        DispatcherConnection(connection, self.mainwindow, self.callback)


    def handle_close(self):
        self.close()

#################################################################################################



class PytronixGuiWin_Base:
    def __init__(self, mainFrame, name = 'not_named'):
        self.name           = name
        self. mainFrame     = mainFrame
        self.proportion     = 0
        self.verticalWxObj  = 0
        self.sizerIndex     = 0
        self.visibleMode    = 0
        self.visibleOn      = 1
        self.function       = self.doNothing1
        self.function2      = self.doNothing2
        
    def doNothing1(self,  event ):
        pass
        #self. mainFrame.logger( 'Do Nothing 1' )

    def doNothing2(self,  event ):
        pass
        #self. mainFrame.logger( 'Do Nothing 2' )
    
    def deSelectAll(self):
        pass



class PytronixGuiWin_List( PytronixGuiWin_Base ):
    def __init__(self, mainFrame, name = 'not_named'):
        PytronixGuiWin_Base.__init__ (self, mainFrame, name )
        self.proportion     = 10
        self.headers        = []
        self.widths         = []
        self.selections     = {}
        self.rowData        = []
        self.selectFunction = 0
        self.lastSelected   = -1
        self.last_data      = []


    def itemSelected(self,  event ):
        self.lastSelectedItem = event.GetItem()
        self.lastSelected = event.GetIndex()
        self.selections[ self.lastSelected ] = 1
        if 0:
            print 'itemSel %-10s %d %s'%( self.name, self.lastSelected, str(self.selections) )
        
        if self.selectFunction:
            self.selectFunction( event )

    def itemDeSelected(self,  event ):
        self.selections[ event.GetIndex() ] = 0
        if self.selectFunction:
            self.selectFunction( event )


    #def getSelectedIndices( self, state =  wx.LIST_STATE_SELECTED):
    #    indices = []
    #    found = 1
    #    lastFound = -1
    #    while found:
    #        index = self.GetNextItem(
    #            lastFound,
    #            wxLIST_NEXT_ALL,
    #            state,
    #            )
    #        if index == -1:
    #            break
    #        else:
    #            lastFound = index
    #            indices.append( index )
    #    return indices

    def getItemByIndex( self, indexIn ):
        found = 1
        lastFound = -1
        state =  wx.LIST_STATE_DONTCARE
        if indexIn < 0:
            indexIn = 0
        for i in range(0,indexIn):
            index = self.wxListCtrl.GetNextItem(
                lastFound,
                wx.LIST_NEXT_ALL,
                state,
                )
            if index == -1:
                break
            else:
                lastFound = index
        return index

    def selectByIndex(self, indexIn ):
        item = self.getItemByIndex(indexIn+1)
        if item != -1:
            self.wxListCtrl.SetItemState( item, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED  )
 
    

    def deSelectAll(self):
        self.selections = {}


    def build(self, do_not_add= 0, otherPanel=0, h_proportion= (1,1) ):
        if not self.verticalWxObj:
            self.wxId = self.mainFrame.idcounter
            self.mainFrame.idcounter += 1
            self.wxListCtrl = wx.ListCtrl(self.mainFrame, self.wxId, style=wx.LC_REPORT|wx.SUNKEN_BORDER|wx.LC_EDIT_LABELS  )
            wx.EVT_LIST_ITEM_SELECTED    (self.mainFrame, self.wxId, self.itemSelected )
            wx.EVT_LIST_ITEM_DESELECTED  (self.mainFrame, self.wxId, self.itemDeSelected )
            if not len( self.headers ):
                self.mainFrame.log.error( 'len( headers ) is zero.' )
            if len( self.headers ) != len( self.widths ) :
                self.mainFrame.log.error( 'len( headers ) != len( widths )' )
            self.headerSize = len( self.headers )
            for i, head in enumerate( self.headers ):
                self.wxListCtrl.InsertColumn( i, head )
            for i, wid in enumerate( self.widths ):
                self.wxListCtrl.SetColumnWidth( i, wid )
            self.verticalWxObj = self.wxListCtrl
            if otherPanel:
                self.verticalWxObj = wx.BoxSizer(wx.HORIZONTAL)
                self.verticalWxObj.Add( otherPanel     , h_proportion[0], wx.EXPAND)
                self.verticalWxObj.Add( self.wxListCtrl, h_proportion[1], wx.EXPAND)
            if not do_not_add:
                self.mainFrame.verticalList.append( self )

    def clear_selected(self):
        if len(self.last_data):
            self.fillText(self.last_data)
        
    def fillText (self, data, maximum = 0 ):
        self.last_data = data
        self.wxListCtrl.DeleteAllItems()
        if 0:
            self.wxListCtrl.ClearAll()        
            for i, head in enumerate( self.headers ):
                self.wxListCtrl.InsertColumn( i, head )
            for i, wid in enumerate( self.widths ):
                self.wxListCtrl.SetColumnWidth( i, wid )
        self.selections = {}
        self.rowData = []
        self.lastSelected = -1
        for i, row in enumerate( data ):
            self.rowData.append( row )
            for j, text in enumerate( row ):
                if j >= self.headerSize:
                    continue
                if maximum and j >= maximum:
                    continue
                if not j:
                    self.wxListCtrl.InsertStringItem(i,    text )
                else    :
                    self.wxListCtrl.SetStringItem   (i, j, text )
        self.wxListCtrl.Refresh()
        
    
    def getSelectedIndices( self, state =  wx.LIST_STATE_SELECTED):
            indices = []
            found = 1
            lastFound = -1
            while found:
                    index = self.wxListCtrl.GetNextItem(
                            lastFound,
                            wx.LIST_NEXT_ALL,
                            state,
                    )
                    if index == -1:
                            break
                    else:
                            lastFound = index
                            indices.append( index )
            return indices    
    

    def set_alternate_colors (self, block_size, colors ):
        item = -1
        loop = 1
        while loop:
            for color in colors:
                for b in range(0,block_size):
                    item = self.wxListCtrl.GetNextItem(item, wx.LIST_NEXT_ALL)
                    if item == -1:
                        loop = 0
                        break
                    self.wxListCtrl.SetItemBackgroundColour(item, color)
                if not loop:
                    break
                
    def set_colors_by_index(self, index_in, normal_color, index_color, error_dict={}, error_color=0, error_color2=0  ):
        item = -1
        index_count = 0
        loop = 1
        while loop:
            item = self.wxListCtrl.GetNextItem(item, wx.LIST_NEXT_ALL)
            if item >= 0:
                isError = index_count in error_dict
                isIndex = index_in == index_count
                if isError and isIndex:
                    self.wxListCtrl.SetItemBackgroundColour(item, error_color2)
                elif isError:
                    self.wxListCtrl.SetItemBackgroundColour(item, error_color)
                elif isIndex:
                    self.wxListCtrl.SetItemBackgroundColour(item, index_color)
                else:
                    self.wxListCtrl.SetItemBackgroundColour(item, normal_color)
                if index_in == index_count:
                    self.wxListCtrl.EnsureVisible(item)
            index_count += 1
            if item == -1:
                loop = 0
                break


    def getFirstSelectedItem(self):
        data = self.getSelectedItems()
        if len( data ):
            return data[0]
        return 0


    def getSelectedItems(self):
        data = []
        rows = self.selections.keys()
        if len( rows ):
            rows.sort()
            for r in rows:
                if self.selections[ r ]:
                    if r >= len(self.rowData):
                        print '\n\n Error in getSelectedItems  r = %d'%(r)
                        for i, rowdata in enumerate( self.rowData ):
                            print '  %3d %s'%( i, str(rowdata) )
                    else:
                        data.append( self.rowData[r] )
        return data

    def getFirstSelectedIndex(self):
        data = self.getSelectedIndexes()
        if len( data ):
            return data[0]
        return -1


    def getSelectedIndexes(self):
        data = []
        rows = self.selections.keys()
        if len( rows ):
            rows.sort()
            for r in rows:
                if self.selections[ r ]:
                    data.append( r )
        return data

    def set_column_text(self, index_in, text_in):
        header = self.wxListCtrl.GetColumn ( index_in )
        header.SetText ( text_in )
        self.wxListCtrl.SetColumn ( index_in, header )


class PytronixGuiWin_Button:
    def __init__(self, mainFrame, parent, x, y, z ):
        self.x            = x
        self.y            = y
        self.z            = z
        self.mainFrame    = mainFrame
        self.parent       = parent
        self.isCheckmark  = False
        self.isRadio      = False
        self.checkmark    = 0
        self.label        = ''
        self.wxObj        = 0
        self.function     = self.parent.doNothing1
        self.wxId         = 0



    def build(self):
        if not self.wxObj:
            self.wxId = self.mainFrame.idcounter
            self.mainFrame.idcounter += 1
            if not self.label:
                #self.label = 'id %3d'%(self.wxId)
                self.label = ' '
            if self.isCheckmark:
                self.wxObj = wx.CheckBox( self.mainFrame, self.wxId, self.label,
                    style = wx.CHK_3STATE | wx.CHK_ALLOW_3RD_STATE_FOR_USER )
                #self.wxObj.SetValue( self.checkmark )
                #self.wxObj.SetValue( self.checkmark )
                wx.EVT_CHECKBOX (self.mainFrame, self.wxId, self.function )
            elif self.isRadio:
                self.wxObj = wx.RadioBox( self.mainFrame, self.wxId, self.label )
                self.wxObj.SetValue( self.checkmark )
                wx.EVT_RADIOBUTTON (self.mainFrame, self.wxId, self.function,
                    style = wx.RB_SINGLE )
            else:
                self.wxObj = wx.Button( self.mainFrame, self.wxId, self.label )
                wx.EVT_BUTTON   (self.mainFrame, self.wxId, self.function )
        #print('i %d %d %d'%(self.x,self.y,self.z))
        return self



class PytronixGuiWin_Buttons( PytronixGuiWin_Base ):
    def __init__(self, mainFrame, sizeX, sizeY, sizeZ=1, commom_binding=0 ):
        PytronixGuiWin_Base.__init__ (self, mainFrame )
        self.id_mapping    = {}
        self.array         = []
        self.sizeX         = sizeX
        self.sizeY         = sizeY
        self.sizeZ         = sizeZ
        for z in range( 0, self.sizeZ  ):
            xx = []
            for x in range( 0, self.sizeX  ):
                yy = []
                for y in range( 0, self.sizeY ):
                    button_obj = PytronixGuiWin_Button( self.mainFrame, self, x, y, z )
                    if self.sizeZ > 1 and x == 0:
                        button_obj.isCheckmark = True
                        #button_obj.isRadio = True
                    if commom_binding:
                        button_obj.function = commom_binding
                    yy.append( button_obj )
                xx.append( yy )
            self.array.append( xx )
        self.color_blue1  = wx.Colour( 230, 230, 255 )
        self.color_blue2  = wx.Colour( 190, 190, 255 )


    def build(self):
        #for z in range( 0, self.sizeZ  ):
        #    for x in range( 0, self.sizeX  ):
        #        for y in range( 0, self.sizeY ):
        #            self.array[z][x][y].build()
        self.verticalWxObj = wx.BoxSizer(wx.VERTICAL)
        for y in range( 0, self.sizeY  ):
            sizerX = wx.BoxSizer(wx.HORIZONTAL)
            for x in range( 0, self.sizeX ):
                button = self.array[0][x][y].build()
                self.id_mapping[button.wxId] = (x,y)
                sizerX.Add( button.wxObj, 1, wx.EXPAND)
            self.verticalWxObj.Add( sizerX, 0, wx.EXPAND)
        self.mainFrame.verticalList.append( self )


    def selectBank(self, bank ):
        if self.sizeZ == 1:
            return
        if bank >= self.sizeZ:
            self.mainFrame.log.error( 'bank(%d) >= self.sizeZ(%d)'%(
                bank,self.sizeZ) )
            return
        for y in range( 0, self.sizeY  ):
            for x in range( 1, self.sizeX ):
                button0 = self.array[   0][x][y]
                button1 = self.array[bank][x][y]
                button0.wxObj.SetLabel( button1.label )
                if button0.isCheckmark:
                    self.mainFrame.log.error( 
                        'selectBank does not support Checkmarks' )
                else:
                    wx.EVT_BUTTON   (self.mainFrame, button0.wxId, button1.function )

        if bank < self.sizeY:
            for y in range( 0, self.sizeY  ):
                button0 = self.array[   0][0][y]
                if y == bank:
                    button0.wxObj.SetBackgroundColour( self.color_blue2 )
                    button0.wxObj.Set3StateValue(2)
                    #button0.wxObj.SetValue(True)
                else:
                    button0.wxObj.SetBackgroundColour( self.color_blue1 )
                    button0.wxObj.Set3StateValue(0)
                    #button0.wxObj.SetValue(False)
                button0.wxObj.Refresh()


    def update_bank_0(self):
        for y in range( 0, self.sizeY  ):
            for x in range( 0, self.sizeX ):
                button = self.array[   0][x][y]
                button.wxObj.SetLabel( button.label )
                if button.isCheckmark:
                    log.error( 'Checkmarks, only use for Z=1' )
                else:
                    wx.EVT_BUTTON   (self.mainFrame, button.wxId, button.function )


    def update_headers(self):
        for y in range( 0, self.sizeY  ):
            button0 = self.array[   0][0][y]
            button0.wxObj.SetLabel( button0.label )
            button0.wxObj.Refresh()



class PytronixGuiWin_LogOutput( PytronixGuiWin_Base ):
    def __init__(self, mainFrame, proportion=5, linkLogToThisObject=1 ):
        PytronixGuiWin_Base.__init__ (self, mainFrame )
        self.proportion          = proportion
        self.pointSize           =  0
        self.linkLogToThisObject =  linkLogToThisObject


    def build(self):
        if not self.verticalWxObj:
            self.wxId   = self.mainFrame.idcounter
            self.mainFrame.idcounter += 1
            self.verticalWxObj = wx.TextCtrl( self.mainFrame, self.wxId, style=wx.TE_READONLY | wx.TE_MULTILINE)
            if self.pointSize:
                self.wxFont = wx.Font( self.pointSize, wx.TELETYPE, wx.NORMAL, wx.NORMAL)
                self.verticalWxObj.SetFont(self.wxFont )
            if self.linkLogToThisObject:
                self.mainFrame.logOutput = self
            self.mainFrame.verticalList.append( self )


class PytronixGuiWin_Text( PytronixGuiWin_Base ):
    def __init__(self, mainFrame ):
        PytronixGuiWin_Base.__init__ (self, mainFrame )
        self.pointSize    = 12
        self.labels       = []
        self.wxTexts      = []
        self.cBackground1 = wx.Colour( 240, 240, 240 )
        self.cBlue        = wx.Colour(  50,  60, 250 )
        self.cRed         = wx.Colour( 250,  60,  50 )
        self.cBlack       = wx.Colour(   0,   0,   0 )
        self.cGreen       = wx.Colour(  50, 250,  50 )


    def build(self):
        if not self.verticalWxObj:
            self.wxId = self.mainFrame.idcounter
            self.mainFrame.idcounter += 1
            if not len( self.labels ):
                self.labels = [ 'id %3d'%(self.wxId) ]
            self.labelCount = len( self.labels )
            self.wxFont       = wx.Font( self.pointSize, wx.SWISS, wx.NORMAL, wx.NORMAL)
            self.verticalWxObj = wx.BoxSizer(wx.HORIZONTAL)
            for label in self.labels:
                staticText = wx.StaticText(self.mainFrame, self.wxId, label, style=wx.ST_NO_AUTORESIZE )
                staticText.SetFont(self.wxFont )
                staticText.SetBackgroundColour( self.cBackground1 )                
                staticText.SetForegroundColour( self.cBlue        )                
                self.wxTexts.append( staticText )
                self.verticalWxObj.Add( staticText, 1, wx.EXPAND)
            self.mainFrame.verticalList.append( self )



#http://wiki.wxpython.org/DoubleBufferedDrawing

class PytronixGuiWin_Image( PytronixGuiWin_Base ):
    def __init__(self, mainFrame, image_filename_in=None, width=100, height=100 ):
        PytronixGuiWin_Base.__init__ (self, mainFrame )
        self.image_filename_in = image_filename_in
        self.image_filename    = -1
        self.width             = width
        self.height            = height
        self.blank_bitmap      = wx.ArtProvider.GetBitmap(wx.ART_NEW, wx.ART_TOOLBAR, (24, 24))
        
        
    def clear_image(self):
        self.static_bitmap.SetBitmap(self.blank_bitmap)
        self.static_bitmap.ClearBackground()


    def reload_image(self, image_filename_in):
        if not image_filename_in:
            self.clear_image()
            return 0
        self.static_bitmap.ClearBackground()
        if  self.image_filename == image_filename_in:
            self.static_bitmap.SetBitmap(self.image_bitmap)
            return 0
        if not os.path.exists(image_filename_in):
            self.mainFrame.log.error( 'Image file does not exists: %s'%(image_filename_in))
            self.clear_image()
            return 1
        self.image_filename  = image_filename_in
        self.image_bitmap    = wx.Image(self.image_filename, wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.static_bitmap.SetBitmap(self.image_bitmap)


    def build(self, do_not_add= 0):
        if not self.verticalWxObj:
            self.wxId = self.mainFrame.idcounter
            self.mainFrame.idcounter += 1
            self.verticalWxObj = wx.BoxSizer(wx.HORIZONTAL)

            self.static_bitmap = wx.StaticBitmap(self.mainFrame, -1, self.blank_bitmap)
            #(-1,-1),(-1,-1)
            #(10 + self.Image.GetWidth(), 5), (self.Image.GetWidth(), self.Image.GetHeight()))
            self.verticalWxObj.Add( self.static_bitmap, 1, wx.EXPAND)
            self.reload_image(self.image_filename_in)
            if not do_not_add:
                self.mainFrame.verticalList.append( self )
        
        notes = '''


        self.clear_image()
        #if imageFile == self.imageFile and not self.image_cleared:
        #    # This image already loaded
        #    return 0
        self.imageFile    = imageFile
        if not os.path.exists(self.imageFile):
            self.mainFrame.log.error( 'Can not load image file: %s'%(self.imageFile) )
            return 1
        self.Image = wx.Image(self.imageFile, wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.staticBitmap.SetBitmap(self.Image)
        self.image_cleared = False
        return 0




            if self.imageFile and os.path.exists(self.imageFile):
                self.Image = wx.Image(self.imageFile, wx.BITMAP_TYPE_ANY).ConvertToBitmap()
                self.image_cleared = False
            else:
                self.Image = self.get_blank_bitmap() 
                self.image_cleared = True
            self.staticBitmap = wx.StaticBitmap(self.mainFrame, -1, self.Image, 
                (10 + self.Image.GetWidth(), 5), (self.Image.GetWidth(), self.Image.GetHeight()))
            self.verticalWxObj.Add( self.staticBitmap, 1, wx.EXPAND)
            if not do_not_add:
                self.mainFrame.verticalList.append( self )
        '''

##########################################################################################################
# Main Class

class PytronixGuiWin(wx.Frame):
    
    def __init__(self,parent,id,title, PosIn=(-1,-1), SizeIn=(-1,-1), log_in=0 ):
        global global_gui
        wx.Frame.__init__(self,parent,wx.ID_ANY, title)
        #self.log          = logging.getLogger('Pytronix.guiWin' )
        self.myPos        = PosIn
        self.mySize       = SizeIn
        self.idcounter    = 100
        self.verticalList = []
        self.dispatcher   = None
        self.logOutput    = None

        global log
        log = log_in
        if not log:
            log = logging.getLogger("PytronixGuiWin")
            log.addHandler(NullLogHandler.NullLogHandler())
        
        self.server_call_modes = [
            'tOpen',
            'tRead',
            'tClose'
            ]
        global_gui = self
        #PytronixStyle.make_tag_attributes( self, self.server_call_modes )        

    #def buildStaticText (self, data ):
    #    #stext = wx.StaticText(self, self.idcounter ,data[iTITLE], style=wx.ALIGN_CENTRE |wx.ST_NO_AUTORESIZE  )
    #    #stext = wx.StaticText(self, self.idcounter ,data[iTITLE], style=wx.ST_NO_AUTORESIZE | wx.SUNKEN_BORDER  )
    #    stext = wx.StaticText(self, self.idcounter ,data[iTITLE], style=wx.ST_NO_AUTORESIZE )
    #    self.idcounter += 1
    #    font = wx.Font( data[iPOINT], wx.SWISS, wx.NORMAL, wx.NORMAL)
    #    stext.SetFont(font)
    #    self.sizer.Add(stext,0,wx.EXPAND)
    #    return stext


    def allVisibleOff (self):
        for item in self.verticalList:
            item.visibleOn = 0
    

    def showVisible (self):
        changeMade = 0
        for item in self.verticalList:
            if item.visibleMode:
                item.deSelectAll()
                self.sizer.Show( item.sizerIndex, item.visibleOn )
                changeMade = 1
        if changeMade:
            self.sizer.Layout()

    
    def pop_last_panel(self):
        if len(self.verticalList):
            panel = self.verticalList[-1]
            self.verticalList = self.verticalList[:-1]
            return panel
        return None

    
    def buildDone (self):
        self.sizer=wx.BoxSizer(wx.VERTICAL)
        for i, item in enumerate( self.verticalList ):
            self.sizer.Add( item.verticalWxObj, item.proportion, wx.EXPAND)
            item.sizerIndex = i
        self.SetSizer(self.sizer)
        self.SetSize( wx.Size( self.mySize[0],self.mySize[1]) )
        self.Layout()
        self.showVisible()
        self.Show(1)


    def LoadDirDialog ( self, defPath ="/", alt_title='' ):
        thedir = ''
        title = "Choose a directory:"
        if alt_title:
            title = alt_title
        dlg = wx.DirDialog(self, title, defaultPath=defPath,
                          style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON)
        if dlg.ShowModal() == wx.ID_OK:
            thedir = dlg.GetPath()
        dlg.Destroy()
        return thedir


    def LoadFileDialog ( self, defPath ="/" ):
        theFile = ''
        dlg = wx.FileDialog(self, "Choose a file:", defaultDir=defPath,
                          style=wx.FD_DEFAULT_STYLE | wx.FD_CHANGE_DIR )
        if dlg.ShowModal() == wx.ID_OK:
            theFile = dlg.GetPath()
        dlg.Destroy()
        return theFile



    def DialogAreYouSure(self, body, title ):
        dlg = wx.MessageDialog(self, body, title, wx.YES_NO | wx.ICON_QUESTION)
        ret = dlg.ShowModal()
        dlg.Destroy()
        return( ret == wx.ID_YES )


    def DialogOK(self, body, title ):
        dlg = wx.MessageDialog(self, body, title, wx.OK )
        ret = dlg.ShowModal()
        dlg.Destroy()
        return( True )


    def TextEntryDialog(self, body, title, initial ):
        dlg = wx.TextEntryDialog(
                self, body,
                title, 'Python')
        dlg.SetValue( initial )
        text = ''
        if dlg.ShowModal() == wx.ID_OK:
            text = dlg.GetValue()
        dlg.Destroy()
        return( text )


    def SingleChoiceDialog(self, body, title, initial, choices ):
        dlg = wx.SingleChoiceDialog(
                self, body,
                title, choices)
        dlg.SetSelection( initial )
        text = ''
        if dlg.ShowModal() == wx.ID_OK:
            text = dlg.GetStringSelection()
        dlg.Destroy()
        return( text )

    def logger_update (self):
        self.logOutput.verticalWxObj.Refresh(1)
        self.logOutput.verticalWxObj.Update()

    def truncate_to_7_bits(self, text_in):
        text_out = ''
        for cc in text_in:
            text_out += chr( ord(cc) & 0x7f )
        return(text_out)

    def logger (self, text_in ):
        if self.logOutput:
            text_in = self.truncate_to_7_bits(text_in)
            text = 'Odd char in text'
            try:
                text = unicode(text_in, "utf-8")
            except:
                pass
            self.logOutput.verticalWxObj.SetInsertionPointEnd()
            self.logOutput.verticalWxObj.WriteText('%s' % text)
        
        

    def loggerClear (self):
        self.logOutput.verticalWxObj.Clear()
        

    ##############################################################################################
    # Server Code
    # From http://wiki.wxpython.org/AsynchronousSockets

    def open_server(self, host, port, callback):
        self.callback = callback
        self.dispatcher = DispatcherServer(host, port, self, callback)
        if not self.dispatcher.is_socke_open:
            return 1
        self.poller = wx.Timer(self, wx.NewId())
        self.Bind(wx.EVT_TIMER, self.poll_server)
        #poll 5 times/second, should be enough for low-bandwidth apps
        self.poller.Start(200, wx.TIMER_CONTINUOUS)

        return 0
            
            
    def poll_server(self, evt):
        asyncore.poll(timeout=0)
        
    def on_close_window(self):
        if self.dispatcher:
            self.dispatcher.close()
            self.dispatcher = None          
            time.sleep(1)
        self.Close(True)  # Close the frame.
        self.exitCall() # Call back to main
        return 0


    ##############################################################################################

    #def OnExit(self,e):
    #    self.Close(True)  # Close the frame.
    #    self.exitCall() # Call back to main

class PytronixGuiOutputHandler(logging.Handler): 
    def __init__(self): 
        logging.Handler.__init__(self) 
        self.textbox = None 
        self.buffer = [] 
        self.gui_win = None 
  
    def set_gui_win(self, gui_win): 
        """ 
        textbox presents the instance of textctrl in the mainframe 
        Method is called after frame is up and running 
        """ 
        self.gui_win = gui_win 

    def shouldFlush(self): 
        if self.gui_win: 
            return True 
        else: 
            return False 
    
    def emit(self, record): 
        self.buffer.append(record) 
        if self.shouldFlush(): 
            self.flush() 

    def flush(self): 
        """ 
        Thread safe flush method which schedule the 
        textbox update after a wx main loop ends 
        """ 
        if self.gui_win:
            for record in self.buffer: 
                msg = self.format(record) 
                #wx.CallAfter(self.textbox.AppendText, msg+'\n') 
                self.gui_win.logger(msg+'\n') 
            self.buffer = [] 

    def close(self): 
        self.flush() 
        self.gui_win = None 
        logging.Handler.close(self) 
    




